/*
 * PmodHYGRO.c
 *
 *  Created on: Feb 11, 2018
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"

#include "PmodHYGRO.h"

#define PMODHYGRO_ADDRESS 0x80

I2C_HandleTypeDef pmodHygroI2c;

void PmodHYGRO_Config()
{
  pmodHygroI2c.Instance = I2C2;
  pmodHygroI2c.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  pmodHygroI2c.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  pmodHygroI2c.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  pmodHygroI2c.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  pmodHygroI2c.Init.OwnAddress1 = 0x01;

  // Set the I2C_TIMINGR register to fulfill the HDC1080 timing requirements (max 400kHz).
  pmodHygroI2c.Init.Timing = 0x10563046;

  HAL_I2C_Init(&pmodHygroI2c);

  // The HDC1080 requires at most 15 ms start-up delay.
  HAL_Delay(15);
}

void HAL_I2C_MspInit(I2C_HandleTypeDef *hi2c)
{
  __HAL_RCC_I2C2_CLK_ENABLE();
  __HAL_RCC_GPIOF_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF4_I2C2;
  GPIO_InitStruct.Pin = GPIO_PIN_0 | GPIO_PIN_1;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
}

void PmodHYGRO_GetMeasurements(int16_t* temperature, int16_t* humidity)
{
  // Write the address 0x00 pointing to temperature register in order to execute the measurements.
  uint8_t data[4] = {0};
  HAL_I2C_Master_Transmit(&pmodHygroI2c, PMODHYGRO_ADDRESS, data, 1, 100);

  // Wait for the measurements to complete.
  HAL_Delay(13);

  // Read the four bytes starting from address 0. First two bytes contain the 14-bit temperature
  // value, the last two bytes - humidity value.
  HAL_I2C_Master_Receive(&pmodHygroI2c, PMODHYGRO_ADDRESS, data, 4, 100);
  uint16_t temperatureRaw = (data[0] << 8) | data[1];
  uint16_t humidityRaw = (data[2] << 8) | data[3];

  // Calculate the real temperature and humidity values according to the formula described on the
  // website: https://reference.digilentinc.com/reference/pmod/pmodhygro/reference-manual
  float temp = (temperatureRaw * 1.0 / (1 << 16) * 165) - 40;
  *temperature = temp * 10;

  temp = (humidityRaw * 1.0 / (1 << 16)) * 100;
  *humidity = temp;
}
